#include "ASCIIarray.h"

// Array of ASCII data.. auto growing,
// designed to dump snatcher text by Artemio Urbina
// aurbina@junkerhq.net

ascii_array new_ascii()
{
	ascii_array text;

	text = (ascii_array)malloc(sizeof(struct ascii_array_st));
	if(!text)
	{
		printf("Out of memory\n");
		exit(1);
	}
	text->address = 0;
	text->array = (int *)malloc(sizeof(int)*BLOCKSIZE);
	if(!text->array)
	{
		free(text);	//lets be clean after all =P
		printf("Out of memory\n");
		exit(1);
	}
	text->csize = 0;
	text->size = BLOCKSIZE;
	text->invalid = 0;
	return(text);
}

ascii_array insert_ascii(int value, ascii_array text)
{
	if(!text)
		text = new_ascii();
	if((text->csize + 1) >= text->size)
		text = grow_ascii(text);
	text->array[text->csize++] = value;
	return(text);
}

void fprint_ascii(FILE *file, ascii_array text)
{
	int i;

	if(!text || !file || !text->csize)
		return;
	if(text->invalid)
		fprintf(file, "*");
	if(text->address)
		fprintf(file, "0x%x: ", text->address);
	for(i = 0; i < text->csize; i++)
	{
		switch(text->array[i])
		{
		case ENTER:
			fprintf(file, "<nl>");
			break;
		case ENTERNL:
			fprintf(file, "\n\t");
			break;
		case COLOR_CHANGE_1:
			{
				fprintf(file, "<cc 1>");
				break;
			}
		case COLOR_CHANGE_2:
			{
				fprintf(file, "<cc 2>");
				break;
			}
		case COLOR_CHANGE_3:
			{
				fprintf(file, "<cc 3>");
				break;
			}
		case DEFAULT_COLOR:
			{
				fprintf(file, "</cc>");
				break;
			}
		case UNKNOWN:
			{
				int a, b;
				
				a = text->array[++i];
				b = text->array[++i];
				fprintf(file, "<0x%x%x>", a, b);
				break;
			}
		default:
			fprintf(file, "%c", text->array[i]);
		}
	}
	fputc('\n', file);
}

ascii_array free_ascii(ascii_array text)
{
	if(!text)
		return(text);
	free(text->array);
	free(text);
	return(NULL);
}

ascii_array grow_ascii(ascii_array text)
{
	int nsize;

	if(!text)
		return(text);
	nsize = text->size + BLOCKSIZE;
	text->array = (int*)realloc(text->array, sizeof(int)*nsize);
	if(!text->array)
	{
		printf("Out of memory\n"); // Cant be clean here....
		free(text);		// a bit at least...
		exit(1);
	}
	text->size += BLOCKSIZE;
	return(text);
}

ascii_array setaddress(int address, ascii_array text)
{
	if(!text)
		text = new_ascii();
	text->address = address;
	return(text);
}
