#include <stdio.h>
#include <time.h>

#include "list_rank.h"
#include "ASCIIarray.h"


// Artemio Urbina Garcia 2001, Snatcher SCD text dumper (for Sega CD)

void printuse();
void extract(char *File, int start, int end, FILE *dump, int pbold);

int main(int argc, char *argv[])
{
        float value;
	FILE *dump;
	clock_t prin, fin;
	int pbold, i, start;
	l_rank ranks, trank;

        prin = clock();

	pbold = 1;
	if(argc < 2)
		printuse();

	start = 1;
	if(!strcmp(argv[1], "-b"))
	{
		pbold = 0;
		start = 2;
	}
	
	for(i = start; i < argc; i ++)
	{
		int len;
		char buffer[1024];

		len = strlen(argv[i]);
		if(len >= 1024)
		{
			printf("Argument too large\n");
			exit(1);
		}

		ranks = new_rank();
		sprintf(buffer, "%s", argv[i]);
		buffer[len - 3] = 'r';
		buffer[len - 2] = 'n';
		buffer[len - 1] = 'k';
		ranks = read_ranks(buffer);

		buffer[len - 3] = 't';
		buffer[len - 2] = 'x';
		buffer[len - 1] = 't';
		if(!strcmp(buffer, argv[i]))
		{
			printf("Source equals destination\n");
			exit(1);
		}
		dump = fopen(buffer, "w");
		if(!dump)
		{
			printf("Could not open file %s\n", buffer);
			exit(1);
		}
		trank = ranks;

		printf("Extracting to %s ", buffer);
		fprintf(dump, "Snatcher Sega CD Dumper\n");
		fprintf(dump, "By: Artemio Urbina (aurbina@junkerhq.net)\n");
		fprintf(dump, "Visit http://junkerhq.net for more stuff\n");
		fprintf(dump, "\n");

		
		if(!ranks)
		{
			printf("No ranks file found... dumping all\n");
			extract(argv[i], 0, 0xfffffff, dump, pbold);
		}
		else
		{
			printf("Ranks file found... dumping\n");
			while(trank)
			{
				extract(argv[i], trank->lower,
					trank->upper, dump, pbold);
				trank = trank->next;
			}
			ranks = free_ranks(ranks);
		}
	
		fclose(dump);
	}
        fin = clock();
        value = (float)(fin - prin)/CLOCKS_PER_SEC;
	printf("Process time: %g seconds\n", value);
	return(0);
}

void printuse()
{
	printf("Use: extract {-b} *.bin\n");
	exit(1);
}

void extract(char *File, int start, int end, FILE *dump, int pbold)
{
	int c, intext, pos, rank;
	FILE *file;
	l_rank	list;
	ascii_array array;

	pos = 0;
	intext = 0;
	c = 'A';
	list = new_rank();
	array = new_ascii();

	file = fopen(File, "rb");
	if(!file)
	{
		printf("Could not open file %s (Copy it from CD)\n", File);
		exit(1);
	}

	if(fseek(file, sizeof(char)*start, SEEK_SET) != 0)
	{
		fclose(dump);
		fclose(file);
		printf("Could not set start offset");
		exit(1);
	}
	list = read_ranks("snatcherscd.rnk");
	pos = start;
	do
	{
		c = fgetc(file);
		pos ++;
		rank = InsideRank(c, list);
		if(rank)
		{
			if(!intext)
			{
				array = free_ascii(array);
			}
			switch(rank)
			{
				case END_OF_STR:
					if(!intext)
						break;
					intext = 0;
					fprint_ascii(dump, array);
					array = free_ascii(array);
					break;
				case ASCII:
					if(!intext)
					{
						intext = 1;
						array = setaddress(pos - 1, array);
					}
					array = insert_ascii(c, array);
					break;
				case DEFAULT_COLOR:
					if(pbold)
						array = insert_ascii(DEFAULT_COLOR, array);
					break;
				case COLOR_CHANGE_1:
				case COLOR_CHANGE_2:
				case COLOR_CHANGE_3:
					if(!intext)
					{
						intext = 1;
						array = setaddress(pos - 1, array);
					}
					if(pbold)
						array = insert_ascii(rank, array);	
					break;
				case ENTER:
					if(pbold)
						array = insert_ascii(ENTER, array);	
					else
						array = insert_ascii(ENTERNL, array);	
					break;
					
			}	
		}
		else
		{
			intext = 0;
			array = free_ascii(array);
		}
	}
	while(c != EOF && pos <= end);
	fclose(file);
	list = free_ranks(list);
}
